/*****************************************************************************
*
*  JdbcConnectTestServlet
*
*****************************************************************************/

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.Statement;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.Vector;

import HTMLPage;

/***********************************************************************
* The JdbcConnectTestServlet class is a simple servlet that can be used to
* test the servlet engine's JDCB configuration.
* <p>
* The HTTP POST used to trigger this servlet should contain the
* following parameter keywords.
* <ul>
* <li> CLASS - JDBC diver class to be loaded dynamically
* <li> DATABASE - JDBC URL for the database connection
* <li> USERID - Database user ID authorized to connect to the database
* <li> PASSWORD - Password used to authenticate the user ID
* <li> SQL - The SQL Select statement to be executed
* </ul>
* <p>
***********************************************************************/
public class JdbcConnectTestServlet
   extends HttpServlet {

   public static final String KEY_CLASS     = "CLASS";
   public static final String KEY_DATABASE  = "DATABASE";
   public static final String KEY_OWNER     = "OWNER";
   public static final String KEY_USERID    = "USERID";
   public static final String KEY_PASSWORD  = "PASSWORD";
   public static final String KEY_SQL       = "SQL";

   /********************************************************************
   * This method is called by the servlet manager after the servlet
   * class is loaded and the servlet object is created but before
   * any post requests are passed to it.
   * <p>
   * For this servlet, it simply invokes the super's method.
   *
   * @param     argConfiguration - The servlet's configuration object
   * @exception ServletException - Any error results in this exception
   ********************************************************************/
   public void init(
     ServletConfig argConfiguration
   )  throws ServletException {
      super.init(argConfiguration);
   }


   /********************************************************************
   * This method is called by the servlet manager every time an HTTP
   * GET request is sent to this servlet.
   * <p>
   * For this servlet, it simply calls this servlets doPost() method.
   *
   * @param     argRequest - The servlet request that triggered this
   * call.
   * @param     argResponse - The servlet response used to reply.
   * @exception ServletException - processing errors throw this
   * exception.
   * @exception IOException - I/O errors throw this exception.
   ********************************************************************/
   public void doGet(
      HttpServletRequest  argRequest,
      HttpServletResponse argResponse
   ) throws ServletException, IOException{

      this.doPost(
         argRequest,
         argResponse
      );

   }

   /********************************************************************
   * This method is called by the servlet manager every time an HTTP
   * POST request is sent to this servlet.
   * <p>
   * For this servlet, it uses the CGI parameters passed on the POST
   * request to
   * <ul>
   * <li> Dynamically load the JDBC class and construct a driver instance
   * <li> Connect to the database
   * <li> Execute the provided SQL statement
   * <li> Display the result
   * </ul>
   *
   * @param     argRequest - The servlet request that triggered this
   * call.
   * @param     argResponse - The servlet response used to reply.
   * @exception ServletException - processing errors throw this
   * exception.
   * @exception IOException - I/O errors throw this exception.
   ********************************************************************/
   public void doPost(
      HttpServletRequest  argRequest,
      HttpServletResponse argResponse
   ) throws ServletException, IOException{


      argResponse.setContentType("text/html");
      PrintWriter varClient = argResponse.getWriter();
      ByteArrayOutputStream varLog = new ByteArrayOutputStream();

      try {

         // Verify that there is only one value for each key because
         // subsequent logic assumes that this is true.
         Enumeration varKeys = argRequest.getParameterNames();
         while (varKeys.hasMoreElements()) {
            String varKey = (String) varKeys.nextElement();
            String[] varValues = argRequest.getParameterValues(varKey);
            if (varValues.length > 1) {
               throw new Exception(
                  "HTML Form Parameter Defect:\n\n" +
                  varKey + " has more than one value.\n"
               );
            }
         }

         // Get the class name.
         String varClass = this.getCgiFormParameter(
            argRequest,
            KEY_CLASS
         );
         if (varClass == null) {
            throw new Exception(
               "HTML Form Parameter Defect:\n\n" +
               KEY_CLASS + " parameter value has not been provided.\n"
            );
         }

         // Get the JDBC URL.
         String varDatabase = this.getCgiFormParameter(
            argRequest,
            KEY_DATABASE
         );
         if (varDatabase == null) {
            throw new Exception(
               "HTML Form Parameter Defect:\n\n" +
               KEY_DATABASE + " parameter value has not been provided.\n"
            );
         }

         // Get the User ID
         String varUserId = this.getCgiFormParameter(
            argRequest,
            KEY_USERID
         );
         if (varUserId == null) {
            throw new Exception(
               "HTML Form Parameter Defect:\n\n" +
               KEY_USERID + " parameter value has not been provided.\n"
            );
         }

         // Get the password.
         String varPassword = this.getCgiFormParameter(
            argRequest,
            KEY_PASSWORD
         );
         if (varPassword == null) {
            throw new Exception(
               "HTML Form Parameter Defect:\n\n" +
               KEY_PASSWORD + " parameter value has not been provided.\n"
            );
         }

         // Get the SQL statement.
         String varSqlStatement = this.getCgiFormParameter(
            argRequest,
            KEY_SQL
         );
         if (varSqlStatement == null) {
            throw new Exception(
               "HTML Form Parameter Defect:\n\n" +
               KEY_SQL + " parameter value has not been provided.\n"
            );
         }

         // Load the JDBC driver class and create an instance to
         // assure that it registers itself with the SQL Driver Manager.
         Object varJdbcDriver = Class.forName(varClass).newInstance();

         // Create a print stream that saves the JDBC driver log in
         // a byte array that can be displayed to the tester.
         DriverManager.setLogStream(new PrintStream(varLog));
         DriverManager.println("");

         // Connect to the database.
         DriverManager.println("JdbcConnectTestServlet:  About to connect");
         Connection varConnection = DriverManager.getConnection(
            varDatabase,
            varUserId,
            varPassword
         );
         DriverManager.println("JdbcConnectTestServlet:  Successfully connected");

         // Create a statement.
         DriverManager.println("JdbcConnectTestServlet:  About to create statement");
         Statement varStatement = varConnection.createStatement();
         DriverManager.println("JdbcConnectTestServlet:  Successfully created statment");

         // Execute the SQL statement provided.
         DriverManager.println("JdbcConnectTestServlet:  About to execute statement");
         boolean varResultSetProduced = varStatement.execute(varSqlStatement);
         DriverManager.println("JdbcConnectTestServlet:  Successfully executed statment");

         // If the SQL statement produced a result set (in other words,
         // it was some sort of query request), process the result set.
         // The results will be saved in a vector of string arrays.
         Vector varResults = new Vector();
         if (varResultSetProduced) {
            DriverManager.println("JdbcConnectTestServlet:  About to process result set");

            // Get the result set.
            ResultSet varResultSet = varStatement.getResultSet();

            // Get the meta data and extract the number of columns and
            // the column names. Add the column name array to the vector.
            ResultSetMetaData varMetaData = varResultSet.getMetaData();
            int varColumnCount = varMetaData.getColumnCount();
            String[] varColList = new String[varColumnCount];
            for (int varIndex = 0; varIndex < varColList.length; varIndex++) {
               varColList[varIndex] = varMetaData.getColumnName(varIndex + 1);
            }
            varResults.addElement(varColList);

            // For each row in the result, obtain each value as a string,
            // save the value in an array of strings, and add the array
            // to the vector.
            while (varResultSet.next()) {
               String[] varValueList = new String[varColumnCount];
               for (int varIndex = 0; varIndex < varValueList.length; varIndex++) {
                  varValueList[varIndex] = varResultSet.getString(varIndex + 1);
               }
               varResults.addElement(varValueList);
            }

            DriverManager.println("JdbcConnectTestServlet:  Successfully processed result set");

            // Close the result set.
            DriverManager.println("JdbcConnectTestServlet:  About to close result set");
            varResultSet.close();
            DriverManager.println("JdbcConnectTestServlet:  Successfully closed result set");
         }

         // Close the statement.
         DriverManager.println("JdbcConnectTestServlet:  About to close statement");
         varStatement.close();
         DriverManager.println("JdbcConnectTestServlet:  Successfully closed statement");

         // Close the connection.
         DriverManager.println("JdbcConnectTestServlet:  About to close connection");
         varConnection.close();
         DriverManager.println("JdbcConnectTestServlet:  Successfully closed connection");

         // Display an HTML document on the tester's browser.
         this.displayResultToClient(
            varResults,
            varLog.toString(),
            varClient
         );

      } catch (Exception varException) {

         // Display the error on the tester's browser.
         this.displayErrorToClient(
            varException,
            varLog.toString(),
            varClient
         );
      }

   }

   /********************************************************************
   * The servlet uses this method to extract a single keyword value from
   * the servlet request.
   *
   * @param     argRequest - The servlet request.
   * @param     argKey - The parameter key whose value is to be
   * returned.
   * @return    - The first value found for the given keyword or null
   * if the value is an empty string.
   ********************************************************************/
   private String getCgiFormParameter(
      HttpServletRequest  argRequest,
      String              argKey
   ) {
      String[] varValues = argRequest.getParameterValues(argKey);
      String varResult = varValues[0];
      if (varResult.length() == 0) {
         varResult = null;
      }
      return varResult;
   }


   /*******************************************************************
   * This method is used to display an error HTML page on the user's
   * browser.
   *
   * @param     argException - The exception thrown when the error
   * was detected.
   * @param     argLog - The database log up to the point where the
   * exception was thrown
   * @param     argClient - The servlet print stream where the HTML
   * page is to be written
   ********************************************************************/
   private void displayErrorToClient(
      Exception   argException,
      String      argLog,
      PrintWriter argClient
   ) {
      String varMessage = argException.getMessage();

      if (varMessage == null) {
         varMessage = "Internal Error - Programming Defect";
      }

      HTMLPage varPage = new HTMLPage(argClient);

      varPage.StartPage();
      varPage.StartHead();
      varPage.Title("JDBC Connect Test" + " Error Message");
      varPage.EndHead();
      varPage.StartBody("#ffffff");

      varPage.HeadingThree("JDBC Connect Test" + " Error Message");

      varPage.StartTable(0);
      varPage.StartTableRow("left", "top");
      varPage.TableHeader("ERROR: ");
      varPage.TableData(varMessage);
      varPage.EndTableRow();
      varPage.EndTable();
      varPage.Paragraph();
      varPage.Paragraph("<B>Exception:</B>");
      varPage.PreFormatted(argException);
      varPage.Paragraph();
      varPage.Paragraph("<B>Database Log:</B>");
      varPage.PreFormatted(argLog);
      varPage.Paragraph("<B>JVM Classpath:</B>");
      StringTokenizer varPath = new StringTokenizer(
         System.getProperty("java.class.path"),
         System.getProperty("path.separator")
      );
      while (varPath.hasMoreTokens()) {
         varPage.Text(varPath.nextToken());
         varPage.LineBreak();
      }
      varPage.Paragraph("<B>Directory that was current when properties were initialized:</B>");
      varPage.Paragraph(System.getProperty("user.dir"));
      varPage.Paragraph("<B>Home directory:</B>");
      varPage.Paragraph(System.getProperty("user.home"));
      varPage.Paragraph("Note: Some JDBC drivers need access to native " +
         "dynamic link libraries.  If your exception contains a " +
         "\"No suitable driver.\" message, check the environment variable " +
         "that contains the dynamic link library path (PATH on WIN/NT, " +
         "LD_LIBRARY_PATH on AIX and Spark).");
      varPage.Paragraph("Press the 'back button' on your browser and correct " +
                  "the problem");

      varPage.EndBody();
      varPage.EndPage();
   }

   /*******************************************************************
   * This method is used to display the result of a command to the
   * tester's browser.
   *
   * @param     argResults - A vector that is either empty (the SQL
   * statement provided was not a query) or a vector where each
   * element is a string array.  The first element contains the
   * column names.  The subsequent elements (if any) contain the
   * values from a row in the result.
   * @param     argLog - The database log.
   * @param     argClient - A servlet print stream where the HTML
   * page is to be written.
   ********************************************************************/
   private void displayResultToClient(
      Vector      argResults,
      String      argLog,
      PrintWriter argClient
   ) {


      HTMLPage varPage = new HTMLPage(argClient);

      varPage.StartPage();
      varPage.StartHead();
      varPage.Title("JDBC Connect Test" + " Success Message");
      varPage.EndHead();
      varPage.StartBody("#ffffff");

      varPage.HeadingThree("JDBC Connect Test" + " Success Message");

      // If the argument is not empty, display the query results in
      // a table.
      if (argResults.size() > 0) {
         // Start the table.
         varPage.StartTable(3);

         // Obtain an enumeration of string arrays.
         Enumeration varRowList = argResults.elements();

         // Get the first string array (by convention it contains the
         // column names) and display the values using table header tags.
         String[] varColRow = (String[])varRowList.nextElement();
         varPage.StartTableRow("left", "top");
         for (int varIndex = 0; varIndex < varColRow.length; varIndex++) {
            varPage.TableHeader(varColRow[varIndex]);
         }
         varPage.EndTableRow();

         // For each remaining string array in the argument (if any),
         // display the values in the table.
         while (varRowList.hasMoreElements()) {
            varPage.StartTableRow("left", "top");
            String[] varDataRow = (String[])varRowList.nextElement();
            for (int varIndex = 0; varIndex < varDataRow.length; varIndex++) {
               varPage.TableData(varDataRow[varIndex]);
            }
            varPage.EndTableRow();
         }

         // End the table.
         varPage.EndTable();
      }

      // Display the database log and important system properties.
      varPage.Paragraph();
      varPage.Paragraph("<B>Database Log:</B>");
      varPage.PreFormatted(argLog);
      varPage.Paragraph("<B>JVM Classpath:</B>");
      StringTokenizer varPath = new StringTokenizer(
         System.getProperty("java.class.path"),
         System.getProperty("path.separator")
      );
      while (varPath.hasMoreTokens()) {
         varPage.Text(varPath.nextToken());
         varPage.LineBreak();
      }
      varPage.Paragraph("<B>Directory that was current when properties were initialized:</B>");
      varPage.Paragraph(System.getProperty("user.dir"));
      varPage.Paragraph("<B>Home directory:</B>");
      varPage.Paragraph(System.getProperty("user.home"));

      // Display the final action message and end the page.
      varPage.Paragraph("Press the 'back button' on your browser " +
                  "to repeat the operation");
      varPage.EndBody();
      varPage.EndPage();
   }
}
