/*****************************************************************************
*
*  SampleStore
*
*****************************************************************************/

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.Hashtable;

import com.ibm.etill.framework.clientapi.PaymentCommandConstants;
import com.ibm.etill.framework.clientapi.PaymentServerClient;
import com.ibm.etill.framework.clientapi.PaymentServerResponse;
import com.ibm.etill.framework.payapi.FrameworkReturnCodes;

import HTMLPage;

/***********************************************************************
* The SampleStore class is a simple servlet that can be used to
* generate Payment Manage orders using the AcceptPayment command.
* <p>
* The HTTP POST used to trigger this servlet should contain the
* following parameter keywords.
* <ul>
* <li> PAYMENTTYPE
* <li> MERCHANTNUMBER
* <li> AMOUNT
* <li> CURRENCY.
* </ul>
* <p>
* If the parameter keywords EXPIRYMONTH and EXPIRYYEAR are provided, the
* values will be combined and set for the $EXPIRY command keyword.
* <p>
* This servlet will use the date/time to generate an order number.
* <p>
* For any parameter keyword that begins with a $, the keyword and
* its value will be passed to the Payment Manager.
***********************************************************************/
public class SampleStore
   extends HttpServlet
   implements PaymentCommandConstants, FrameworkReturnCodes {

   /********************************************************************
   * This servlet attribute contains the name of the Payment Manager
   * host.  Normally, it would be initialized from configuration
   * (e.g. a servlet parameter).
   ********************************************************************/
   String objPaymentManagerHost = "localhost";
   /********************************************************************
   * This servlet attribute contains the port where the Payment Manager
   * listens.  Normally, it would be initialized from configuration
   * (e.g. a servlet parameter).
   ********************************************************************/
   int    objPaymentManagerPort = 80;
   /********************************************************************
   * This servlet attribute contains the path to XML Document Type
   * Definitions.  Normally, it would be initialized from configuration
   * (e.g. a servlet parameter).
   ********************************************************************/
   String objDtdPath = null;
   /********************************************************************
   * This servlet attribute contains the user ID to be used when
   * connecting.  Normally, it would be initialized from configuration
   * (e.g. a servlet parameter).
   ********************************************************************/
   String objUserId = "admin";
   /********************************************************************
   * This servlet attribute contains the password used to authenticate
   * the user.  Normally, it would be initialized from configuration
   * (e.g. a servlet parameter).
   ********************************************************************/
   String objPassword = "admin";
   /********************************************************************
   * This servlet attribute contains the client object constructed
   * during initializaiton.
   ********************************************************************/
   PaymentServerClient objPaymentManager = null;

   /********************************************************************
   * This method is called by the servlet manager after the servlet
   * class is loaded and the servlet object is created but before
   * any post requests are passed to it.
   * <p>
   * For this servlet, it simply initializes the object used to
   * communicate with the Payment Manager.
   *
   * @param     argConfiguration - The servlet's configuration
   * environment.
   * @exception ServletException - Any error results in this
   * exception.
   ********************************************************************/
   public void init(
                   ServletConfig argConfiguration
                   )  throws ServletException {
      super.init(argConfiguration);

      // Construct client object.
      objPaymentManager = new PaymentServerClient(
         this.objDtdPath,
         this.objPaymentManagerHost,
         this.objPaymentManagerPort
      );
   }


   /********************************************************************
   * This method is called by the servlet manager every time an HTTP
   * POST request is sent to this servlet.
   * <p>
   * For this servlet, it uses the CGI parameters passed on the POST
   * request to build an AcceptPayment request for the Payment Manager.
   * Then it builds a dynamic HTML page as a response.
   *
   * @param     argRequest - The servlet request that triggered this
   * call.
   * @param     argResponse - The servlet response used to reply.
   * @exception ServletException - processing errors throw this
   * exception.
   * @exception IOException - I/O errors throw this exception.
   ********************************************************************/
   public void doPost(
      HttpServletRequest  argRequest,
      HttpServletResponse argResponse
   ) throws ServletException, IOException{


      argResponse.setContentType("text/html");
      PrintWriter varClient = argResponse.getWriter();

      try {

         // Verify that there is only one value for each key because
         // subsequent logic assumes that this is true.
         Enumeration varKeys = argRequest.getParameterNames();
         while (varKeys.hasMoreElements()) {
            String varKey = (String) varKeys.nextElement();
            String[] varValues = argRequest.getParameterValues(varKey);
            if (varValues.length > 1) {
               throw new Exception(
                  "HTML Form Parameter Defect:\n\n" +
                  varKey + " has more than one value.\n"
               );
            }
         }

         // Set the command name.
         String varCommand = OP_ACCEPTPAYMENT;

         // Construct a table to contain the command's parameters.
         Hashtable varParameters = new Hashtable();

         // Create a string reference for convenience.
         String varValue = null;

         // If there is a payment type, set the parameter.
         varValue = this.getServletParameter(
            argRequest,
            KEY_PAYMENTTYPE
         );
         if (varValue != null) {
            varParameters.put(
               KEY_PAYMENTTYPE,
               varValue
            );
         }


         // If there is a merchant number, set the parameter.
         varValue = this.getServletParameter(
            argRequest,
            KEY_MERCHANTNUMBER
         );
         if (varValue != null) {
            varParameters.put(
               KEY_MERCHANTNUMBER,
               varValue
            );
         }

         // Compute and set a parameter for the order number.
         // This allows us to use the same HTML form repeatedly.
         varValue = Integer.toString(
            (int) (System.currentTimeMillis() & 0x0fffffff)
         );
         varParameters.put(
            KEY_ORDERNUMBER,
            varValue
         );


         // If there is an amount, set the parameter.
         varValue = this.getServletParameter(
            argRequest,
            KEY_AMOUNT
         );
         if (varValue != null) {
            varParameters.put(
               KEY_AMOUNT,
               varValue
            );
         }

         // If there is a currency, set the parameter.
         varValue = this.getServletParameter(
            argRequest,
            KEY_CURRENCY
         );
         if (varValue != null) {
            varParameters.put(
               KEY_CURRENCY,
               varValue
            );
         }

         // Build expiry from month and year and set the parameter
         String varMonth = this.getServletParameter(
            argRequest,
            "EXPIRYMONTH"
         );
         String varYear  = this.getServletParameter(
            argRequest,
            "EXPIRYYEAR"
         );
         String varExpiry =
            (varYear  != null ? varYear  : "") +
            (varMonth != null ? varMonth : "");
         if (varExpiry.length() > 0) {
            varParameters.put(
               PD_EXPIRY,
               varExpiry
            );
         }

         // For each key that begins with a $, add the key and its
         // value to the parameter list.
         varKeys = argRequest.getParameterNames();
         while (varKeys.hasMoreElements()) {
            String varKey = (String) varKeys.nextElement();
            if (varKey.startsWith("$")) {
               varValue = this.getServletParameter(
                  argRequest,
                  varKey
               );
               if (varValue != null) {
                  varParameters.put(
                     varKey,
                     varValue
                  );
               }
            }
         }

         PaymentServerResponse varResponse =
         this.objPaymentManager.issueCommand(
            varCommand,
            varParameters,
            this.objUserId,
            this.objPassword
         );

         this.displayResultToClient(
            varResponse.getPrimaryRC(),
            varResponse.getSecondaryRC(),
            varClient
         );


      } catch (Exception varException) {
         this.displayErrorToClient(
            varException,
            varClient
         );
      }

   }

   /********************************************************************
   * The servlet uses this method to extract a single keyword value from
   * the servlet request.
   *
   * @param     argRequest - The servlet request.
   * @param     argKey - The parameter key whose value is to be
   * returned.
   * @return    - The first value found for the given keyword or null
   * if the value is an empty string.
   ********************************************************************/
   private String getServletParameter(
      HttpServletRequest  argRequest,
      String              argKey
   ) {
      String[] varValues = argRequest.getParameterValues(argKey);
      String varResult = varValues[0];
      if (varResult.length() == 0) {
         varResult = null;
      }
      return varResult;
   }


   /*******************************************************************
   * This method is used to display an error HTML page on the user's
   * browser.
   *
   * @param     argException - The exception generated when the error
   * was detected.
   * @param     argClient - A servlet print stream where the HTML
   * page is to be written.
   *******************************************************************/
   private void displayErrorToClient(
      Exception   argException,
      PrintWriter argClient
   ) {
      String varMessage = argException.getMessage();

      if (varMessage == null) {
         varMessage = "Internal Error - Programming Defect";
      }

      HTMLPage varPage = new HTMLPage(argClient);

      varPage.StartPage();
      varPage.StartHead();
      varPage.Title("Sample Store" + " Error Message");
      varPage.EndHead();
      varPage.StartBody("#ffffff");

      varPage.HeadingThree("Sample Store" + " Error Message");

      varPage.StartTable(0);
      varPage.StartTableRow("left", "top");
      varPage.TableHeader("ERROR: ");
      varPage.TableData(varMessage);
      varPage.EndTableRow();
      varPage.EndTable();
      varPage.PreFormatted(argException);
      varPage.Paragraph("Press the 'back button' on your browser and correct " +
                  "the problem");

      varPage.EndBody();
      varPage.EndPage();
   }

   /*******************************************************************
   * This method is used to display the result of a command to the
   * tester's browser.
   *
   * @param     argPrimaryRc - The primary return code returned by
   * the Payment Manager.
   * @param     argSecondaryRc - The secondary return code returned
   * by the Payment Manager.
   * @param     argClient - A servlet print stream where the HTML
   * page is to be written.
   ********************************************************************/
   private void displayResultToClient(
      short       argPrimaryRc,
      short       argSecondaryRc,
      PrintWriter argClient
   ) {

      String varHeader  = "Error:";
      String varMessage = "The requested operation has failed.";
      if (argPrimaryRc == PRC_OPERATION_SUCCESS) {
         if (argSecondaryRc == RC_NONE) {
            varHeader  = "Successful:";
            varMessage = "The requested operation has been completed.";
         } else {
            varHeader  = "Warning:";
            varMessage = "The requested operation has been completed " +
               "but the secondary return code indicates a warning.";
         }
      }

      HTMLPage varPage = new HTMLPage(argClient);

      varPage.StartPage();
      varPage.StartHead();
      varPage.Title("Sample Store" + " Result Message");
      varPage.EndHead();
      varPage.StartBody("#ffffff");

      varPage.HeadingThree("Sample Store" + " Result Message");

      varPage.StartTable(0);
      varPage.StartTableRow("left", "top");
      varPage.TableHeader(varHeader);
      varPage.TableData(varMessage);
      varPage.EndTableRow();
      varPage.StartTableRow("left", "top");
      varPage.TableHeader("Primary Return Code ==>");
      varPage.TableData(Short.toString(argPrimaryRc), "right", "top");
      varPage.EndTableRow();
      varPage.StartTableRow("left", "top");
      varPage.TableHeader("Secondary Return Code ==>");
      varPage.TableData(Short.toString(argSecondaryRc), "right", "top");
      varPage.EndTableRow();
      varPage.EndTable();
      varPage.Paragraph("Press the 'back button' on your browser " +
                  "to repeat the operation");

      varPage.EndBody();
      varPage.EndPage();
   }
}
